/**
 * 
 */
package view;

import java.awt.HeadlessException;

import java.awt.*;
import javax.swing.*;
import model.DotsModelInterface;
import model.DotsModelInterface.TeamColour;
import model.Observer;

/**
 * @author dennis
 *
 */
public class DotsFrame extends JFrame implements Runnable, Observer {

	private boolean mUIReady = false;

	/**
	 * @throws HeadlessException if started without a display.
	 */
	public DotsFrame(DotsModelInterface game) throws HeadlessException {
	    mUIReady = false;
	    mGame = game;
	    javax.swing.SwingUtilities.invokeLater(this);

	}
	
	public void refresh() {
		if (mUIReady) {
			mBScoreText.setText(new Integer(mGame.getScore(TeamColour.BLACK)).toString());
			mWScoreText.setText(new Integer(mGame.getScore(TeamColour.WHITE)).toString());		
			repaint();
			if (!mGame.canPlayContinue()) {
				String msg = "Game Over!";
				
				String winner;
				int wScore = mGame.getScore(TeamColour.WHITE);
				int bScore = mGame.getScore(TeamColour.BLACK);
				if (wScore > bScore) {
					winner = "White wins!";
				} else if (bScore > wScore) {
					winner = "BLACK wins!";
				} else {
					winner = "Tie!";
				}
			    JOptionPane.showMessageDialog(this, winner, msg, JOptionPane.OK_OPTION);
			    mGame.reset();
			} else {
				String msg = mGame.getCurrentTurn() + "'s turn.";
				setStatus(msg);
			}
		}
	}
	
	/**
	 * @return true if UI has been created.
	 */
	public boolean uiIsReady() {
		return mUIReady;
	}
	
	/**
	 * Construct the UI. This approach is used to avoid thread safety problems.
	 * @see http://java.sun.com/docs/books/tutorial/uiswing/misc/threads.html
	 */
	private void createAndShowUI() {
	    setTitle("ENGI 9874 Dots Game");
	    setSize(WIDTH, HEIGHT);
	    mBoard = new BoardView(mGame);
	    Container pane = getContentPane(); 
	    createStatusPanel();
	    pane.add(mStatusPanel, BorderLayout.SOUTH);
	    createScorePanel();
	    pane.add(mScorePanel, BorderLayout.NORTH);
	    pane.add(mBoard, BorderLayout.CENTER);
	    setJMenuBar(createMenus());
	    mListener = new BoardController(this, mBoard, mGame);
	    mBoard.addMouseListener(mListener);
//	    pane.add(createToolBar(), BorderLayout.NORTH);


	    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    setVisible(true);
//	    mBoard.rescale();
		mUIReady = true; // must be last line of method 
	}
	
	public void run() {
		createAndShowUI();
	}
	
	public void setStatus(String txt) {
		mStatusText.setText(txt);
	}
	
	private void createStatusPanel() {
		mStatusPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JLabel status = new JLabel("Status:");
		status.setFont(new Font("Default", Font.BOLD, 16));
		mStatusPanel.add(status);
		mStatusText = new JTextField("", 15);
		mStatusText.setFont(new Font("Default", Font.PLAIN, 16));
		mStatusText.setEditable(false);
		mStatusPanel.add(mStatusText);		
	}
	
	private void createScorePanel() {
		mScorePanel = new JPanel();
		JLabel scoreLabel = new JLabel("Score:");
		scoreLabel.setFont(new Font("Default", Font.BOLD, 16));
		mScorePanel.add(scoreLabel);
		JLabel wLabel = new JLabel("White:");
		wLabel.setFont(new Font("Default", Font.PLAIN, 16));
		mScorePanel.add(wLabel);
		mWScoreText = new JTextField("", 3);
		mWScoreText.setEditable(false);
		mWScoreText.setText("0");
		mScorePanel.add(mWScoreText);

		JLabel bLabel = new JLabel("BLACK:");
		bLabel.setFont(new Font("Default", Font.PLAIN, 16));
		mScorePanel.add(bLabel);
		mBScoreText = new JTextField("", 3);
		mBScoreText.setEditable(false);
		mBScoreText.setText("0");
		mScorePanel.add(mBScoreText);
	
	}
	
    private JMenuBar createMenus() {
	    JMenuBar bar = new JMenuBar();
	    JMenu gameMenu = new JMenu("Game");
	    bar.add(gameMenu);
/*	    openAction = new OpenAction();
	    fileMenu.add(openAction);
	
	    JMenu rallyMenu = new JMenu("Rally");
	    bar.add(rallyMenu);
	    rallyGoAction = new RallyGoAction();
	    rallyMenu.add(rallyGoAction);
	    rallyResetAction = new RallyResetAction();
	    rallyMenu.add(rallyResetAction);
	    rallyAbortAction = new RallyAbortAction();
	    rallyMenu.add(rallyAbortAction);
	*/
	    bar.add(Box.createHorizontalGlue()); // make the following left justified
	    JMenu helpMenu = new JMenu("Help");
	    bar.add(helpMenu);
	    return bar;
	}
    
    /** Textfield for status */
    private JTextField mStatusText;

    /** Textfield for BLACK Team score */
    private JTextField mBScoreText;

    /** Textfield for White Team score */
    private JTextField mWScoreText;

    /** Dots game model. */
	private DotsModelInterface mGame;
	
	/** Panel that shows playing board */
	private BoardView mBoard;
	
	/** Controller for game */
	private BoardController mListener;
	
	/** place for status messages. */
	private JPanel mStatusPanel;
	
	/** place for score fields */
	private JPanel mScorePanel;
	
	/** Initial width of frame. */
	private static final int WIDTH = 410;
	
	/** Initial height of frame. */
	private static final int HEIGHT = 500;

}
